/*--------------------------------------------------*/

#import "ItsBetaVk.h"
#import "ItsBetaVkUser.h"

/*--------------------------------------------------*/
#if defined(TARGET_OS_IPHONE)
/*--------------------------------------------------*/

#import "NSString+ItsBeta.h"
#import "NSDictionary+ItsBeta.h"
#import "NSURL+ItsBeta.h"

/*--------------------------------------------------*/

#define ITS_BETA_REQUEST_PERMISSIONS_URL @"https://oauth.vk.com/authorize?client_id=%@&scope=%@&redirect_uri=%@&display=touch&response_type=token"

/*--------------------------------------------------*/

@interface ItsBetaVk () {
    NSString* _accessToken;
    NSDate* _expirationDate;
    NSMutableDictionary< ItsBetaVkUser >* _user;
    NSError* _lastError;
    NSString* _errorString;
}

- (void) close;

@end

/*--------------------------------------------------*/

@implementation ItsBetaVk

- (id) init {
    self = [super init];
    if(self != nil) {
        _user = NS_SAFE_RETAIN([NSMutableDictionary vkUser]);
        _clientId = NS_SAFE_RETAIN(@"3469447");
        _redirectURL = NS_SAFE_RETAIN(@"https://oauth.vk.com/blank.html");
        NSArray* scope = [NSArray arrayWithObjects:@"wall", @"offline", nil];
        _scope = NS_SAFE_RETAIN(scope);
        _isHavePermissions = YES;
        _animated = YES;
    }
    return self;
}

- (void) dealloc {
    NS_SAFE_RELEASE(_webView);
    NS_SAFE_RELEASE(_activityIndicator);
    NS_SAFE_RELEASE(_parentController);
    
    NS_SAFE_RELEASE(_clientId);
    NS_SAFE_RELEASE(_redirectURL);
    NS_SAFE_RELEASE(_scope);
    
    NS_SAFE_RELEASE(_accessToken);
    NS_SAFE_RELEASE(_expirationDate);
    NS_SAFE_RELEASE(_user);
    NS_SAFE_RELEASE(_lastError);
        
#if !__has_feature(objc_arc)
    [super dealloc];
#endif
}

- (void) viewDidLoad {
    [super viewDidLoad];
    
    [self request];
}

- (void) viewDidUnload {
    [self setWebView:nil];
    [self setActivityIndicator:nil];
    [self setParentController:nil];

    [super viewDidUnload];
}

- (void) loadView {
    [super loadView];
    
    CGRect viewFrame = [[self view] frame];
    [[self view] setBackgroundColor:[UIColor whiteColor]];
    [[self view] setAutoresizesSubviews:YES];
    
    /*------------ Navbar ------------*/
    UINavigationBar* navBar = [[UINavigationBar alloc] initWithFrame:CGRectMake(0, 0, viewFrame.size.width, 44.0f)];
    [navBar setTranslucent:NO];
    [navBar setTintColor:[UIColor darkGrayColor]];
    [navBar setAutoresizingMask:(UIViewAutoresizingFlexibleWidth)];
    UINavigationItem* navItem = [[UINavigationItem alloc] initWithTitle:@"Достижения"];
    [navItem setRightBarButtonItem:[[UIBarButtonItem alloc] initWithTitle:@"Закрыть" style:UIBarButtonItemStyleBordered target:self action:@selector(closePressed:)]];
    [navBar setItems:@[navItem]];
    [[self view] addSubview:navBar];
    NS_SAFE_RELEASE(navItem);
    NS_SAFE_RELEASE(navBar);
    /*--------------------------------*/
    
    
    /*------------ Webview ------------*/
    UIWebView* webView = [[UIWebView alloc] initWithFrame:CGRectMake(0, 44, viewFrame.size.width, viewFrame.size.height - 44)];
    [webView setAutoresizingMask:(UIViewAutoresizingFlexibleWidth|UIViewAutoresizingFlexibleHeight)];
    [webView setScalesPageToFit:YES];
    [webView setBackgroundColor:[UIColor whiteColor]];
    [webView setDelegate:self];
    [[self webView] setAutoresizesSubviews:YES];
    [[self view] addSubview:webView];
    [self setWebView:webView];
    NS_SAFE_RELEASE(webView);
    /*---------------------------------*/
    
    /*------------ Activity indicator ------------*/
    UIActivityIndicatorView* actInd = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
    [actInd setCenter:CGPointMake(CGRectGetMidX(viewFrame), CGRectGetMidY(viewFrame))];
    [actInd setAutoresizingMask:(UIViewAutoresizingFlexibleLeftMargin|UIViewAutoresizingFlexibleRightMargin|UIViewAutoresizingFlexibleBottomMargin)];
    [actInd setHidesWhenStopped:YES];
    [actInd setHidden:YES];
    [[self view] addSubview:actInd];
    [self setActivityIndicator:actInd];
    NS_SAFE_RELEASE(actInd);
    /*--------------------------------------------*/
}

#pragma mark - Public methods

- (void) request {
    if([self isViewLoaded] == YES) {
        NSString* url = [NSString stringWithFormat:ITS_BETA_REQUEST_PERMISSIONS_URL, _clientId, [_scope componentsJoinedByString:@","], _redirectURL];
        if(url != nil) {
            [[self webView] loadRequest:[NSURLRequest requestWithURL:[NSURL URLWithString:url]]];
        }
    } else {
        [self view];
    }
}

#pragma mark - Private methods

- (void) close {
    if([_parentController presentedViewController] == self)
    {
        if(SYSTEM_VERSION_LESS_THAN(@"5.0")) {
            [_parentController dismissModalViewControllerAnimated:[self animated]];
            [self execCallback];
        } else {
            [_parentController dismissViewControllerAnimated:[self animated] completion:^{
                [self execCallback];
            }];
        }
    }
    else
    {
        [self execCallback];
    }
}

- (void) execCallback
{
    if(([_accessToken length] > 0) && ([_user count] > 0)) {
        if(_successCallback != nil) {
            _successCallback(_accessToken, _expirationDate, _user, _isHavePermissions);
        }
    } else {
        if(_failureCallback != nil) {
            if(_lastError == nil)
            {
                NSError* error = [NSError errorWithDomain:ItsBetaErrorDomain
                                                     code:ItsBetaErrorFacebookAuth
                                                 userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
                                                           NSLocalizedString(@"Vk.Fail.Session", @""), NSLocalizedDescriptionKey,
                                                           nil]];
                NS_SAFE_SETTER(_lastError, error);
            }
            _failureCallback(_lastError);
        }
    }
    NS_SAFE_RELEASE(_accessToken);
    NS_SAFE_RELEASE(_expirationDate);
    NS_SAFE_RELEASE(_lastError);
    NS_SAFE_RELEASE(_parentController);
}

#pragma mark - IBActions

- (IBAction) closePressed:(id)sender {
    [self close];
}

#pragma mark - UIWebViewDelegate

- (BOOL) webView:(UIWebView*)webView shouldStartLoadWithRequest:(NSURLRequest*)request navigationType:(UIWebViewNavigationType)type {
    if([[[request URL] absoluteString] hasPrefix:@"https://login.vk.com"] == YES) {
        NSDictionary* components = [[request URL] queryComponents];
        NSString* cancelled = [[components objectForKey:@"cancel"] objectAtIndex:0];
        if([cancelled isEqualToString:@"1"] == YES) {
            [self close];
            return NO;
        }
    }
    
    if(type == UIWebViewNavigationTypeLinkClicked) {
        [[UIApplication sharedApplication] openURL:[request URL]];
        return NO;
    }
    
    if([[[request URL] absoluteString] hasPrefix:_redirectURL] == YES) {
        NSDictionary* components = [[request URL] queryComponents];
        if(components == nil)
        {
            components = [[request URL] fragmentComponents];
        }
        NS_SAFE_SETTER(_errorString, [[components objectForKey:@"error"] objectAtIndex:0]);
        NS_SAFE_SETTER(_accessToken, [[components objectForKey:@"access_token"] objectAtIndex:0]);
        NSString* uid = [[components objectForKey:@"user_id"] objectAtIndex:0];
        NSTimeInterval tokenLifeTime = [[[components objectForKey:@"expires_in"] objectAtIndex:0] doubleValue];
        if(tokenLifeTime == 0)
            tokenLifeTime = 31536000;   // 1 year
        NS_SAFE_SETTER(_expirationDate, [NSDate dateWithTimeIntervalSinceNow:tokenLifeTime]);
        if(_accessToken != nil && uid != nil && _expirationDate != nil) {
            [_activityIndicator startAnimating];
            NSMutableString *profileRequestUrl = [[NSMutableString alloc] init];
            [profileRequestUrl appendFormat:@"%@/", @"https://api.vk.com/method"];
            [profileRequestUrl appendFormat:@"%@?", @"getProfiles"];
            [profileRequestUrl appendFormat:@"uid=%@&", uid];
            NSMutableString *fields = [[NSMutableString alloc] init];
            [fields appendString:@"screen_name,bdate"];
            [profileRequestUrl appendFormat:@"fields=%@&", fields];
            [profileRequestUrl appendFormat:@"access_token=%@", _accessToken];
            NSURLRequest* profileRequest = [NSURLRequest requestWithURL:[NSURL URLWithString:profileRequestUrl]];
            [NSURLConnection sendAsynchronousRequest:profileRequest
                                               queue:[NSOperationQueue mainQueue]
                                   completionHandler:^(NSURLResponse* response, NSData* data, NSError* error) {
                                       if(error == nil) {
                                           NSDictionary* profile = nil;
                                           NSDictionary* responseDictionary = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
                                           if(error == nil && [responseDictionary isKindOfClass:[NSDictionary class]] == YES) {
                                               NSArray* profiles = [responseDictionary objectForKey:@"response"];
                                               if(profiles != nil && [profiles isKindOfClass:[NSArray class]] == YES) {
                                                   profile = [profiles objectAtIndex:0];
                                                   if(profile != nil && [profile isKindOfClass:[NSDictionary class]] == YES) {
                                                       [_user setValuesForKeysWithDictionary:profile];
                                                   }
                                               }
                                           }
                                       }
                                       if(error != nil) {
                                           NS_SAFE_SETTER(_lastError, error);
                                       }
                                       [_activityIndicator stopAnimating];
                                       [self close];
                                   }];
        }
        else if(_errorString != nil)
        {
            [self close];
        }
        return NO;
    }
    return YES;
}

- (void) webViewDidStartLoad:(UIWebView*)webView {
    [_activityIndicator startAnimating];
}

- (void) webViewDidFinishLoad:(UIWebView*)webView {
    NSString* content = [webView stringByEvaluatingJavaScriptFromString:@"document.body.innerHTML"];
    if([content length] > 0) {
        if([_parentController presentedViewController] != self) {
            if(SYSTEM_VERSION_LESS_THAN(@"5.0")) {
                [_parentController presentModalViewController:self animated:[self animated]];
            } else {
                [_parentController presentViewController:self animated:[self animated] completion:nil];
            }
        }
    }
    [_activityIndicator stopAnimating];
}

- (void)webView:(UIWebView*)webView didFailLoadWithError:(NSError*)error {
    if([error code] == NSURLErrorCancelled) {
        if([_parentController presentedViewController] != self) {
            if(SYSTEM_VERSION_LESS_THAN(@"5.0")) {
                [_parentController presentModalViewController:self animated:[self animated]];
            } else {
                [_parentController presentViewController:self animated:[self animated] completion:nil];
            }
        }
        return;
    }
    if(error.code == 102 && [error.domain isEqual:@"WebKitErrorDomain"]) {
        return;
    }
    if(error != nil) {
        NS_SAFE_SETTER(_lastError, error);
    }
    [self close];
}

- (BOOL) shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation {
    if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        return YES;
    else
        return NO;
}

- (BOOL) shouldAutorotate {
    return YES;
}

- (NSUInteger)supportedInterfaceOrientations {
    if(UI_USER_INTERFACE_IDIOM() == UIUserInterfaceIdiomPad)
        return UIInterfaceOrientationMaskAll;
    else
        return UIInterfaceOrientationMaskPortrait;
}

@end

/*--------------------------------------------------*/
#endif
/*--------------------------------------------------*/
