/*--------------------------------------------------*/

#import "ItsBetaObject.h"

/*--------------------------------------------------*/

#if defined(ITSBETA_USE_FACEBOOK_OFFICIAL_SDK)
#   import <FacebookSDK/FacebookSDK.h>
#else
#   import "ItsBetaGraphUser.h"
#endif

#import "ItsBetaVkUser.h"

/*--------------------------------------------------*/

typedef void (^ItsBetaPlayerLogin)(NSError* error);
typedef void (^ItsBetaPlayerLogout)(NSError* error);
typedef void (^ItsBetaPlayerCreateAchievement)(NSString* activateCode, NSError* error);
typedef void (^ItsBetaPlayerActivateAchievement)(NSString* object_id, NSError* error);
typedef void (^ItsBetaPlayerGiveAchievement)(NSString* object_id, NSError* error);

/*--------------------------------------------------*/

@interface ItsBetaPlayer : NSObject< NSCoding >

@property(nonatomic, readwrite) ItsBetaPlayerType type; // Тип пользователя
@property(nonatomic, readwrite, assign) NSString* typeString; // Тип пользователя в виде строки
@property(nonatomic, readonly) NSString* Id; // Уникальный Id пользователя
@property(nonatomic, readonly) NSString* socialId; // Id в соц. сети
@property(nonatomic, readonly) NSString* socialToken; // access token в соц. сети
@property(nonatomic, readonly) NSDate* socialTokenExpirationDate;   // дата и время истечения токена
@property(nonatomic, readonly) NSDictionary <ItsBetaGraphUser>* graphUser; // Объект пользователя соц. сети (fb)
@property(nonatomic, readonly) NSDictionary <ItsBetaVkUser>* vkUser; // Объект пользователя соц. сети (vk)
@property(nonatomic, readonly) BOOL isHavePermissions;

@property(nonatomic, readonly) ItsBetaObjectCollection* objects;

+ (ItsBetaPlayer*) playerWithType:(ItsBetaPlayerType)type;

- (id) initWithType:(ItsBetaPlayerType)type;

- (BOOL) synchronizeWithProject:(ItsBetaProject*)project;

- (BOOL) isLogined;
- (BOOL) isLoginedInFacebook;
- (BOOL) isLoginedInVk;

#if defined(TARGET_OS_IPHONE)
- (void) loginWithViewController:(UIViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback;
#else
- (void) loginWithViewController:(NSViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback;
#endif
- (void) loginWithFacebookId:(NSString*)facebookId facebookToken:(NSString*)facebookToken expirationDate:(NSDate*)expirationDate graphUser:(NSDictionary<ItsBetaGraphUser>*)graphUser isHavePermissions:(BOOL)isHavePermissions callback:(ItsBetaPlayerLogin)callback;
- (void) logout:(ItsBetaPlayerLogout)callback;

- (void) loginWithVkId:(NSString*)vkId vkToken:(NSString*)vkToken expirationDate:(NSDate*)expirationDate vkUser:(NSDictionary<ItsBetaVkUser>*)vkUser isHavePermissions:(BOOL)isHavePermissions callback:(ItsBetaPlayerLogin)callback;

- (void) createAchievementWithProject:(ItsBetaProject*)project objectTemplate:(ItsBetaObjectTemplate*)objectTemplate params:(NSDictionary*)params callback:(ItsBetaPlayerCreateAchievement)callback;
- (void) activateAchievementWithProject:(ItsBetaProject*)project activateCode:(NSString*)activateCode callback:(ItsBetaPlayerActivateAchievement)callback;
- (void) giveAchievementWithProject:(ItsBetaProject*)project objectTemplate:(ItsBetaObjectTemplate*)objectTemplate params:(NSDictionary*)params callback:(ItsBetaPlayerGiveAchievement)callback;

+ (BOOL) handleOpenURL:(NSURL*)url;

@end

/*--------------------------------------------------*/
