/*--------------------------------------------------*/

#import "ItsBetaPlayer.h"
#import "ItsBetaFastCache.h"
#import "ItsBetaRest.h"
#import "ItsBetaQueue.h"
#import "ItsBetaApi.h"
#import "ItsBeta.h"

/*--------------------------------------------------*/

#if defined(ITSBETA_USE_FACEBOOK_OFFICIAL_SDK)
#   import <FacebookSDK/FacebookSDK.h>
#else
#   import "ItsBetaFacebook.h"
#endif

#import "ItsBetaVk.h"

/*--------------------------------------------------*/

#define ITSBETA_PLAYER_OBJECT_DEFAULT_PAGE_SIZE     100

/*--------------------------------------------------*/

@interface ItsBetaPlayer () {
    NSDate* _lastUpdateObjects;
    
#if defined(ITSBETA_USE_FACEBOOK_OFFICIAL_SDK)
#else
    ItsBetaFacebook* _facebookController;
#endif
    ItsBetaVk* _vkController;
}

#if defined(ITSBETA_USE_FACEBOOK_OFFICIAL_SDK)

- (void) loginWithFacebookOfficialSDK:(ItsBetaPlayerLogin)callback;
- (void) logoutWithFacebookOfficialSDK:(ItsBetaPlayerLogout)callback;

#else

#if defined(TARGET_OS_IPHONE)
- (void) loginFacebookWithViewController:(UIViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback;
#else
- (void) loginFacebookWithViewController:(NSViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback;
#endif
- (void) logoutWithFacebook:(ItsBetaPlayerLogout)callback;

#endif

#if defined(TARGET_OS_IPHONE)
- (void) loginVkWithViewController:(UIViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback;
#else
- (void) loginVkWithViewController:(NSViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback;
#endif
- (void) logoutWithVk:(ItsBetaPlayerLogout)callback;

@end

/*--------------------------------------------------*/

@implementation ItsBetaPlayer

- (void) setType:(ItsBetaPlayerType)type {
    if(_type != type) {
        [self logout:nil];
        
        _type = type;
    }
}

- (void) setTypeString:(NSString*)typeString {
    [self setType:[ItsBetaApi playerTypeFromString:typeString]];
}

- (NSString*) typeString {
    return [ItsBetaApi playerTypeToString:_type];
}

+ (ItsBetaPlayer*) playerWithType:(ItsBetaPlayerType)type {
    return NS_SAFE_AUTORELEASE([[self alloc] initWithType:type]);
}

- (id) initWithCoder:(NSCoder*)coder {
    self = [super init];
    if(self != nil) {
        _type = [coder decodeIntForKey:@"type"];
        _lastUpdateObjects = NS_SAFE_RETAIN([coder decodeObjectForKey:@"last_update_objects"]);
        _objects = NS_SAFE_RETAIN([coder decodeObjectForKey:@"objects"]);
        _Id = NS_SAFE_RETAIN([coder decodeObjectForKey:@"player_id"]);
        _socialId = NS_SAFE_RETAIN([coder decodeObjectForKey:@"social_id"]);
        _socialToken = NS_SAFE_RETAIN([coder decodeObjectForKey:@"social_token"]);
        _socialTokenExpirationDate = NS_SAFE_RETAIN([coder decodeObjectForKey:@"social_token_expiration_date"]);
        _graphUser = NS_SAFE_RETAIN([coder decodeObjectForKey:@"facebook_graph_user"]);
        _vkUser = NS_SAFE_RETAIN([coder decodeObjectForKey:@"vk_user"]);
        _isHavePermissions = [coder decodeBoolForKey:@"is_have_permissions"];
    }
    return self;
}

- (id) initWithType:(ItsBetaPlayerType)type {
    self = [super init];
    if(self != nil) {
        _type = type;
        _objects = NS_SAFE_RETAIN([ItsBetaObjectCollection collection]);
        _isHavePermissions = NO;
    }
    return self;
}

- (void) dealloc {
    NS_SAFE_RELEASE(_Id);
    NS_SAFE_RELEASE(_socialId);
    NS_SAFE_RELEASE(_socialToken);
    NS_SAFE_RELEASE(_socialTokenExpirationDate);
    NS_SAFE_RELEASE(_objects);
    NS_SAFE_RELEASE(_graphUser);
    NS_SAFE_RELEASE(_vkUser);
    
#if defined(ITSBETA_USE_FACEBOOK_OFFICIAL_SDK)
#else
    NS_SAFE_RELEASE(_facebookController);
#endif
    
#if !__has_feature(objc_arc)
    [super dealloc];
#endif
}

- (void) encodeWithCoder:(NSCoder*)coder {
    [coder encodeInt:_type forKey:@"type"];
    [coder encodeObject:_lastUpdateObjects forKey:@"last_update_objects"];
    [coder encodeObject:_objects forKey:@"objects"];
    [coder encodeObject:_Id forKey:@"player_id"];
    [coder encodeObject:_socialId forKey:@"social_id"];
    [coder encodeObject:_socialToken forKey:@"social_token"];
    [coder encodeObject:_socialTokenExpirationDate forKey:@"social_token_expiration_date"];
    [coder encodeObject:_graphUser forKey:@"facebook_graph_user"];
    [coder encodeObject:_vkUser forKey:@"vk_user"];
    [coder encodeBool:_isHavePermissions forKey:@"is_have_permissions"];
}

- (BOOL) synchronizeWithProject:(ItsBetaProject*)project {
    if([ItsBeta applicationAccessToken] == nil) {
        // ERROR
        return NO;
    }
    if(_Id != nil) {
        __block NSUInteger countObjects = 0;
        NSDate* lastUpdateObjects = [NSDate date];
        [ItsBetaApi requestServiceURL:[ItsBeta applicationServiceURL]
                          accessToken:[ItsBeta applicationAccessToken]
                           lastUpdate:_lastUpdateObjects
                               player:self
                              project:project
                         countObjects:^(NSUInteger count, NSError *error) {
                             countObjects = count;
                         }];
        if(countObjects > 0) {
            NSUInteger countPage = 1;
            if(countObjects > ITSBETA_PLAYER_OBJECT_DEFAULT_PAGE_SIZE) {
                countPage = countObjects / ITSBETA_PLAYER_OBJECT_DEFAULT_PAGE_SIZE;
                if((countObjects % ITSBETA_PLAYER_OBJECT_DEFAULT_PAGE_SIZE) != 0) {
                    countPage++;
                }
            }
            for(NSUInteger page = 0; page < countPage; page++) {
                [ItsBetaApi requestServiceURL:[ItsBeta applicationServiceURL]
                                  accessToken:[ItsBeta applicationAccessToken]
                                   lastUpdate:_lastUpdateObjects
                                       player:self
                                      project:project
                                         page:page + 1
                                      prePage:ITSBETA_PLAYER_OBJECT_DEFAULT_PAGE_SIZE
                                      objects:^(ItsBetaObjectCollection* collection, NSError *error) {
                                          [_objects setObjects:collection];
                                      }];
            }
        }
        NS_SAFE_SETTER(_lastUpdateObjects, lastUpdateObjects);
    }
    return YES;
}

- (BOOL) isLogined {
    return ((_socialId != nil) && (_socialToken != nil));
}

- (BOOL) isLoginedInFacebook {
    if(_socialId != nil && _socialToken != nil && _type == ItsBetaPlayerTypeFacebook)
    {
        if([[_socialTokenExpirationDate laterDate:[NSDate date]] isEqualToDate:_socialTokenExpirationDate] == YES)
        {
            return YES;
        }
    }
    return NO;
}

- (BOOL) isLoginedInVk {
    if(_socialId != nil && _socialToken != nil && _type == ItsBetaPlayerTypeVk)
    {
        if([[_socialTokenExpirationDate laterDate:[NSDate date]] isEqualToDate:_socialTokenExpirationDate] == YES)
        {
            return YES;
        }
    }
    return NO;
}

#if defined(TARGET_OS_IPHONE)
- (void) loginWithViewController:(UIViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback {
#else
- (void) loginWithViewController:(NSViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback {
#endif
    [[NSNotificationCenter defaultCenter] postNotificationName:ItsBetaWillPlayerLogin object:self];
    switch(_type) {
        case ItsBetaPlayerTypeFacebook:
#if defined(ITSBETA_USE_FACEBOOK_OFFICIAL_SDK)
            [self loginWithFacebookOfficialSDK:callback];
#else
            [self loginFacebookWithViewController:viewController animated:animated callback:callback];
#endif
        break;
        case ItsBetaPlayerTypeVk:
            [self loginVkWithViewController:viewController animated:animated callback:callback];
        break;
        default:
            [[NSNotificationCenter defaultCenter] postNotificationName:ItsBetaDidPlayerLogin object:self];
            if(callback != nil) {
                callback(nil);
            }
        break;
    }
}

- (void) loginWithFacebookId:(NSString*)facebookId facebookToken:(NSString*)facebookToken expirationDate:(NSDate*)expirationDate graphUser:(NSDictionary<ItsBetaGraphUser>*)graphUser isHavePermissions:(BOOL)isHavePermissions callback:(ItsBetaPlayerLogin)callback {
    [ItsBetaApi requestServiceURL:[ItsBeta applicationServiceURL]
                      accessToken:[ItsBeta applicationAccessToken]
                       facebookId:facebookId
                         playerId:^(NSString* playerId, NSError* error) {
                             if(_Id != playerId) {
                                 NS_SAFE_SETTER(_Id, playerId);
                             }
                             if(_socialId != facebookId) {
                                 NS_SAFE_SETTER(_socialId, facebookId);
                             }
                             if(_socialToken != facebookToken) {
                                 NS_SAFE_SETTER(_socialToken, facebookToken);
                             }
                             if([_socialTokenExpirationDate isEqualToDate:expirationDate] == NO) {
                                 NS_SAFE_SETTER(_socialTokenExpirationDate, expirationDate);
                             }
                             if(_graphUser != graphUser) {
                                 NS_SAFE_SETTER(_graphUser, graphUser);
                             }
                             if(isHavePermissions != _isHavePermissions) {
                                 _isHavePermissions = isHavePermissions;
                             }
                             if((_Id != nil) && (error != nil)) {
                                 callback(error);
                             } else {
                                 callback(nil);
                             }
                             [[NSNotificationCenter defaultCenter] postNotificationName:ItsBetaDidPlayerLogin object:self];
                         }];
}

- (void) loginWithVkId:(NSString*)vkId vkToken:(NSString*)vkToken expirationDate:(NSDate*)expirationDate vkUser:(NSDictionary<ItsBetaVkUser>*)vkUser isHavePermissions:(BOOL)isHavePermissions callback:(ItsBetaPlayerLogin)callback {
    [ItsBetaApi requestServiceURL:[ItsBeta applicationServiceURL]
                      accessToken:[ItsBeta applicationAccessToken]
                             vkId:vkId
                         playerId:^(NSString* playerId, NSError* error) {
                             if(_Id != playerId) {
                                 NS_SAFE_SETTER(_Id, playerId);
                             }
                             if(_socialId != vkId) {
                                 NS_SAFE_SETTER(_socialId, vkId);
                             }
                             if(_socialToken != vkToken) {
                                 NS_SAFE_SETTER(_socialToken, vkToken);
                             }
                             if([_socialTokenExpirationDate isEqualToDate:expirationDate] == NO) {
                                 NS_SAFE_SETTER(_socialTokenExpirationDate, expirationDate);
                             }
                             if(_vkUser != vkUser) {
                                 NS_SAFE_SETTER(_vkUser, vkUser);
                             }
                             if(_isHavePermissions != isHavePermissions) {
                                 _isHavePermissions = isHavePermissions;
                             }
                             if((_Id != nil) && (error != nil)) {
                                 callback(error);
                             } else {
                                 callback(nil);
                             }
                             [[NSNotificationCenter defaultCenter] postNotificationName:ItsBetaDidPlayerLogin object:self];
                         }];
}

- (void) logout:(ItsBetaPlayerLogout)callback {
    [[NSNotificationCenter defaultCenter] postNotificationName:ItsBetaWillPlayerLogout object:self];
    switch(_type) {
        case ItsBetaPlayerTypeFacebook:
#if defined(ITSBETA_USE_FACEBOOK_OFFICIAL_SDK)
            [self logoutWithFacebookOfficialSDK:callback];
#else
            [self logoutWithFacebook:callback];
#endif
        break;
        case ItsBetaPlayerTypeVk:
            [self logoutWithVk:callback];
        break;
        default:
            if(callback != nil) {
                callback(nil);
            }
        break;
    }
    [[NSNotificationCenter defaultCenter] postNotificationName:ItsBetaDidPlayerLogout object:self];
}

- (void) createAchievementWithProject:(ItsBetaProject*)project objectTemplate:(ItsBetaObjectTemplate*)objectTemplate params:(NSDictionary*)params callback:(ItsBetaPlayerCreateAchievement)callback {
    [ItsBetaApi requestServiceURL:[ItsBeta applicationServiceURL]
                      accessToken:[ItsBeta applicationAccessToken]
                          project:project
                   objectTemplate:objectTemplate
                           params:params
                     activateCode:^(NSString* activateCode, NSError* error) {
                         if(callback != nil) {
                             callback(activateCode, error);
                         }
                     }];
}

- (void) activateAchievementWithProject:(ItsBetaProject*)project activateCode:(NSString*)activateCode callback:(ItsBetaPlayerActivateAchievement)callback {
    [ItsBetaApi requestServiceURL:[ItsBeta applicationServiceURL]
                      accessToken:[ItsBeta applicationAccessToken]
                          project:project
                           player:self
                     activateCode:activateCode
                           object:^(NSString* object_id, NSError* error) {
                               if(error == nil) {
                                   if((_Id == nil) && (_type == ItsBetaPlayerTypeFacebook)) {
                                       [self loginWithFacebookId:_socialId facebookToken:_socialToken expirationDate:_socialTokenExpirationDate graphUser:_graphUser isHavePermissions:_isHavePermissions callback:^(NSError *error) {
                                           if(callback != nil) {
                                               callback(object_id, error);
                                           }
                                       }];
                                       return;
                                   }
                                   else if((_Id == nil) && (_type == ItsBetaPlayerTypeVk)) {
                                       [self loginWithVkId:_socialId vkToken:_socialToken expirationDate:_socialTokenExpirationDate vkUser:_vkUser isHavePermissions:_isHavePermissions callback:^(NSError *error) {
                                           if(callback != nil) {
                                               callback(object_id, error);
                                           }
                                       }];
                                       return;
                                   }
                               }
                               if(callback != nil) {
                                   callback(object_id, error);
                               }
                           }];
}

- (void) giveAchievementWithProject:(ItsBetaProject*)project objectTemplate:(ItsBetaObjectTemplate*)objectTemplate params:(NSDictionary*)params callback:(ItsBetaPlayerGiveAchievement)callback {
    [ItsBetaApi requestServiceURL:[ItsBeta applicationServiceURL]
                      accessToken:[ItsBeta applicationAccessToken]
                          project:project
                   objectTemplate:objectTemplate
                           params:params
                           player:self
                           object:^(NSString* object_id, NSError *error) {
                               if(error == nil) {
                                   if((_Id == nil) && (_type == ItsBetaPlayerTypeFacebook)) {
                                       [self loginWithFacebookId:_socialId facebookToken:_socialToken expirationDate:_socialTokenExpirationDate graphUser:_graphUser isHavePermissions:_isHavePermissions callback:^(NSError *error) {
                                           if(callback != nil) {
                                               callback(object_id, error);
                                           }
                                       }];
                                       return;
                                   }
                                   else if((_Id == nil) && (_type == ItsBetaPlayerTypeVk)) {
                                       [self loginWithVkId:_socialId vkToken:_socialToken expirationDate:_socialTokenExpirationDate vkUser:_vkUser isHavePermissions:_isHavePermissions callback:^(NSError *error) {
                                           if(callback != nil) {
                                               callback(object_id, error);
                                           }
                                       }];
                                       return;
                                   }
                               }
                               else
                               {
                                   NSString* errorDescription = [error localizedDescription];
                                   NSRange startPos = [errorDescription rangeOfString:@"{"];
                                   NSRange endPos = [errorDescription rangeOfString:@"}" options:NSBackwardsSearch];
                                   if(startPos.length > 0 && endPos.length > 0)
                                   {
                                       errorDescription = [errorDescription substringWithRange:NSMakeRange(startPos.location, endPos.location - startPos.location + 1)];
                                       NSError* jsonError = nil;
                                       id json = [NSJSONSerialization JSONObjectWithData:[errorDescription dataUsingEncoding:NSUTF8StringEncoding] options:0 error:&jsonError];
                                       if(json != nil && [json isKindOfClass:[NSDictionary class]] == YES)
                                       {
                                           NSDictionary* itsBetaError = [json objectForKey:@"error"];
                                           if(itsBetaError != nil && [itsBetaError isKindOfClass:[NSDictionary class]] == YES)
                                           {
                                               NSInteger code = [[itsBetaError objectForKey:@"code"] integerValue];
                                               if(code == 200)
                                               {
                                                   _isHavePermissions = NO;
                                               }
                                           }
                                       }
                                   }
                                   
                               }
                               if(callback != nil) {
                                   callback(object_id, error);
                               }
                           }];
}

#if defined(ITSBETA_USE_FACEBOOK_OFFICIAL_SDK)

- (void) loginWithFacebookOfficialSDK:(ItsBetaPlayerLogin)callback {
#if __has_feature(objc_arc)
    __weak id safeSelf = self;
#else
    id safeSelf = self;
#endif
    [FBSession openActiveSessionWithPublishPermissions:[NSArray arrayWithObjects:@"publish_stream", @"publish_actions", nil]
                                       defaultAudience:FBSessionDefaultAudienceOnlyMe
                                          allowLoginUI:YES
                                     completionHandler:^(FBSession *session, FBSessionState state, NSError *error) {
                                         switch(state) {
                                             case FBSessionStateOpen:
                                             case FBSessionStateOpenTokenExtended:
                                                 [[FBRequest requestForMe] startWithCompletionHandler:^(FBRequestConnection *connection, NSDictionary< FBGraphUser > *user, NSError *error) {
                                                     if(error == nil) {
                                                         [safeSelf loginWithFacebookId:[user id] facebookToken:[session accessToken] expirationDate:[session expirationDate] graphUser:user callback:callback];
                                                     } else {
                                                         callback([[error userInfo] objectForKey:@"com.facebook.sdk:ErrorInnerErrorKey"]);
                                                     }
                                                 }];
                                                 break;
                                             case FBSessionStateClosedLoginFailed:
                                                 [[FBSession activeSession] closeAndClearTokenInformation];
                                                 error = [NSError errorWithDomain:ItsBetaErrorDomain
                                                                             code:ItsBetaErrorFacebookAuth
                                                                         userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
                                                                                   @"Facebook.Fail.Session", NSLocalizedDescriptionKey,
                                                                                   nil]];
                                                 break;
                                             case FBSessionStateClosed:
                                                 break;
                                             default:
                                                 error = [NSError errorWithDomain:ItsBetaErrorDomain
                                                                             code:ItsBetaErrorFacebookAuth
                                                                         userInfo:[NSDictionary dictionaryWithObjectsAndKeys:
                                                                                   @"Facebook.Fail.Unknown", NSLocalizedDescriptionKey,
                                                                                   nil]];
                                                 break;
                                         }
                                         if(error != nil) {
                                             callback(error);
                                         }
                                     }];
}

- (void) logoutWithFacebookOfficialSDK:(ItsBetaPlayerLogout)callback {
    if([FBSession activeSession] != nil) {
        [[FBSession activeSession] closeAndClearTokenInformation];
    }
    
    NS_SAFE_RELEASE(_Id);
    NS_SAFE_RELEASE(_socialId);
    NS_SAFE_RELEASE(_socialToken);
    NS_SAFE_RELEASE(_socialTokenExpirationDate);
    NS_SAFE_RELEASE(_graphUser);
    NS_SAFE_RELEASE(_vkUser);
    NS_SAFE_RELEASE(_lastUpdateObjects);
    [_objects removeAllObjects];
    
    if(callback != nil) {
        callback(nil);
    }
}

#else

#if defined(TARGET_OS_IPHONE)
- (void) loginFacebookWithViewController:(UIViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback {
     _facebookController = NS_SAFE_RETAIN([ItsBetaFacebook new]);
    if(_facebookController != nil) {
        [_facebookController setParentController:viewController];
    }
#else
- (void) loginFacebookWithViewController:(NSViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback {
#endif
    if(_facebookController != nil) {
#if __has_feature(objc_arc)
        __weak id safeSelf = self;
#else
        id safeSelf = self;
#endif
        [_facebookController setSuccessCallback:^(NSString* accessToken, NSDate* expirationDate, NSDictionary< ItsBetaGraphUser >* user, BOOL isHavePermissions) {
            [safeSelf loginWithFacebookId:[user id] facebookToken:accessToken expirationDate:expirationDate graphUser:user isHavePermissions:isHavePermissions callback:callback];
        }];
        [_facebookController setFailureCallback:^(NSError* error) {
            callback(error);
        }];
        
        [_facebookController setAnimated:animated];
        
        [_facebookController request];
    }
}

- (void) logoutWithFacebook:(ItsBetaPlayerLogout)callback {
    NSHTTPCookieStorage* storage = [NSHTTPCookieStorage sharedHTTPCookieStorage];
    if(storage != nil) {
        for(NSHTTPCookie* cookie in [storage cookies]) {
            NSRange range = [[cookie domain] rangeOfString:@"facebook"];
            if(range.length > 0) {
                [storage deleteCookie:cookie];
            }
        }
        [[NSUserDefaults standardUserDefaults] synchronize];
    }
    
    NS_SAFE_RELEASE(_Id);
    NS_SAFE_RELEASE(_socialId);
    NS_SAFE_RELEASE(_socialToken);
    NS_SAFE_RELEASE(_socialTokenExpirationDate);
    NS_SAFE_RELEASE(_graphUser);
    NS_SAFE_RELEASE(_vkUser);
    NS_SAFE_RELEASE(_lastUpdateObjects);
    [_objects removeAllObjects];
    
    if(callback != nil) {
        callback(nil);
    }
}

#endif
    
#if defined(TARGET_OS_IPHONE)
- (void) loginVkWithViewController:(UIViewController *)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback{
    _vkController = NS_SAFE_RETAIN([ItsBetaVk new]);
    if(_vkController != nil) {
        //[viewController presentViewController:_vkController animated:YES completion:^{}];
        [_vkController setParentController:viewController];
    }
#else
- (void) loginVkWithViewController:(NSViewController*)viewController animated:(BOOL)animated callback:(ItsBetaPlayerLogin)callback {
#endif
    if(_vkController != nil) {
#if __has_feature(objc_arc)
        __weak id safeSelf = self;
#else
        id safeSelf = self;
#endif
        [_vkController setSuccessCallback:^(NSString* accessToken, NSDate* expirationDate, NSDictionary< ItsBetaVkUser >* user, BOOL isHavePermissions) {
            [safeSelf loginWithVkId:[user uid] vkToken:accessToken expirationDate:expirationDate vkUser:user isHavePermissions:isHavePermissions callback:callback];
        }];
        [_vkController setFailureCallback:^(NSError* error) {
            callback(error);
        }];
        
        [_vkController setAnimated:animated];
        
        [_vkController request];
    }
}
    
- (void) logoutWithVk:(ItsBetaPlayerLogout)callback {
    NSHTTPCookieStorage* storage = [NSHTTPCookieStorage sharedHTTPCookieStorage];
    if(storage != nil) {
        for(NSHTTPCookie* cookie in [storage cookies]) {
            NSRange range = [[cookie domain] rangeOfString:@"vk"];
            if(range.length > 0) {
                [storage deleteCookie:cookie];
            }
        }
        [[NSUserDefaults standardUserDefaults] synchronize];
    }
    
    NS_SAFE_RELEASE(_Id);
    NS_SAFE_RELEASE(_socialId);
    NS_SAFE_RELEASE(_socialToken);
    NS_SAFE_RELEASE(_socialTokenExpirationDate);
    NS_SAFE_RELEASE(_graphUser);
    NS_SAFE_RELEASE(_vkUser);
    NS_SAFE_RELEASE(_lastUpdateObjects);
    [_objects removeAllObjects];
    
    if(callback != nil) {
        callback(nil);
    }
}
    
+ (BOOL) handleOpenURL:(NSURL*)url {
#if defined(ITSBETA_USE_FACEBOOK_OFFICIAL_SDK)
    return [[FBSession activeSession] handleOpenURL:url];
#else
    return NO;
#endif
}

@end

/*--------------------------------------------------*/